@@ -20,7 +20,8 @@ import (
 	"github.com/artemiscloud/activemq-artemis-operator/pkg/resources/serviceports"
 	"github.com/artemiscloud/activemq-artemis-operator/pkg/resources/statefulsets"
 	ss "github.com/artemiscloud/activemq-artemis-operator/pkg/resources/statefulsets"
-	"github.com/artemiscloud/activemq-artemis-operator/pkg/utils/cr2jinja2"
+	"github.com/artemiscloud/activemq-artemis-operator/pkg/utils/config"
+	cr2jinja2v2alpha3 "github.com/artemiscloud/activemq-artemis-operator/pkg/utils/cr2jinja2"
 	"github.com/artemiscloud/activemq-artemis-operator/version"
 	"github.com/go-logr/logr"
 	"k8s.io/apimachinery/pkg/api/errors"
@@ -39,6 +40,7 @@ import (
 	"reflect"
 
 	routev1 "github.com/openshift/api/route/v1"
+	extv1b1 "k8s.io/api/extensions/v1beta1"
 
 	logf "sigs.k8s.io/controller-runtime/pkg/runtime/log"
 
@@ -79,31 +81,32 @@ type ActiveMQArtemisReconciler struct {
 }
 
 type ActiveMQArtemisIReconciler interface {
-	Process(customResource *brokerv2alpha3.ActiveMQArtemis, client client.Client, scheme *runtime.Scheme, firstTime bool) uint32
-	ProcessStatefulSet(customResource *brokerv2alpha3.ActiveMQArtemis, client client.Client, log logr.Logger, firstTime bool) (*appsv1.StatefulSet, bool)
+	Process(fsm *ActiveMQArtemisFSM, client client.Client, scheme *runtime.Scheme, firstTime bool) uint32
+	ProcessStatefulSet(fsm *ActiveMQArtemisFSM, client client.Client, log logr.Logger, firstTime bool) (*appsv1.StatefulSet, bool)
 	ProcessCredentials(customResource *brokerv2alpha3.ActiveMQArtemis, client client.Client, scheme *runtime.Scheme, currentStatefulSet *appsv1.StatefulSet) uint32
 	ProcessDeploymentPlan(customResource *brokerv2alpha3.ActiveMQArtemis, client client.Client, scheme *runtime.Scheme, currentStatefulSet *appsv1.StatefulSet, firstTime bool) uint32
 	ProcessAcceptorsAndConnectors(customResource *brokerv2alpha3.ActiveMQArtemis, client client.Client, scheme *runtime.Scheme, currentStatefulSet *appsv1.StatefulSet) uint32
 	ProcessConsole(customResource *brokerv2alpha3.ActiveMQArtemis, client client.Client, scheme *runtime.Scheme, currentStatefulSet *appsv1.StatefulSet)
 	ProcessResources(customResource *brokerv2alpha3.ActiveMQArtemis, client client.Client, scheme *runtime.Scheme, currentStatefulSet *appsv1.StatefulSet) uint8
+	ProcessAddressSettings(customResource *brokerv2alpha3.ActiveMQArtemis, client client.Client) bool
 }
 
-func (reconciler *ActiveMQArtemisReconciler) Process(customResource *brokerv2alpha3.ActiveMQArtemis, client client.Client, scheme *runtime.Scheme, firstTime bool) (uint32, uint8) {
+func (reconciler *ActiveMQArtemisReconciler) Process(fsm *ActiveMQArtemisFSM, client client.Client, scheme *runtime.Scheme, firstTime bool) (uint32, uint8) {
 
 	var log = logf.Log.WithName("controller_v2alpha3activemqartemis")
-	log.Info("Reconciler Processing...", "Operator version", version.Version, "ActiveMQArtemis release", customResource.Spec.Version)
+	log.Info("Reconciler Processing...", "Operator version", version.Version, "ActiveMQArtemis release", fsm.customResource.Spec.Version)
 
-	currentStatefulSet, firstTime := reconciler.ProcessStatefulSet(customResource, client, log, firstTime)
-	statefulSetUpdates := reconciler.ProcessDeploymentPlan(customResource, client, scheme, currentStatefulSet, firstTime)
-	statefulSetUpdates |= reconciler.ProcessCredentials(customResource, client, scheme, currentStatefulSet)
-	statefulSetUpdates |= reconciler.ProcessAcceptorsAndConnectors(customResource, client, scheme, currentStatefulSet)
-	statefulSetUpdates |= reconciler.ProcessConsole(customResource, client, scheme, currentStatefulSet)
+	currentStatefulSet, firstTime := reconciler.ProcessStatefulSet(fsm, client, log, firstTime)
+	statefulSetUpdates := reconciler.ProcessDeploymentPlan(fsm.customResource, client, scheme, currentStatefulSet, firstTime)
+	statefulSetUpdates |= reconciler.ProcessCredentials(fsm.customResource, client, scheme, currentStatefulSet)
+	statefulSetUpdates |= reconciler.ProcessAcceptorsAndConnectors(fsm.customResource, client, scheme, currentStatefulSet)
+	statefulSetUpdates |= reconciler.ProcessConsole(fsm.customResource, client, scheme, currentStatefulSet)
 
 	requestedResources = append(requestedResources, currentStatefulSet)
-	stepsComplete := reconciler.ProcessResources(customResource, client, scheme, currentStatefulSet)
+	stepsComplete := reconciler.ProcessResources(fsm.customResource, client, scheme, currentStatefulSet)
 
 	if statefulSetUpdates > 0 {
-		ssNamespacedName := types.NamespacedName{Name: ss.NameBuilder.Name(), Namespace: customResource.Namespace}
+		ssNamespacedName := types.NamespacedName{Name: ss.NameBuilder.Name(), Namespace: fsm.customResource.Namespace}
 		if err := resources.Update(ssNamespacedName, client, currentStatefulSet); err != nil {
 			log.Error(err, "Failed to update StatefulSet.", "Deployment.Namespace", currentStatefulSet.Namespace, "Deployment.Name", currentStatefulSet.Name)
 		}
@@ -112,19 +115,25 @@ func (reconciler *ActiveMQArtemisReconciler) Process(customResource *brokerv2alp
 	return statefulSetUpdates, stepsComplete
 }
 
-func (reconciler *ActiveMQArtemisReconciler) ProcessStatefulSet(customResource *brokerv2alpha3.ActiveMQArtemis, client client.Client, log logr.Logger, firstTime bool) (*appsv1.StatefulSet, bool) {
+func (reconciler *ActiveMQArtemisReconciler) ProcessStatefulSet(fsm *ActiveMQArtemisFSM, client client.Client, log logr.Logger, firstTime bool) (*appsv1.StatefulSet, bool) {
 
 	ssNamespacedName := types.NamespacedName{
 		Name:      ss.NameBuilder.Name(),
-		Namespace: customResource.Namespace,
+		Namespace: fsm.customResource.Namespace,
 	}
 	currentStatefulSet, err := ss.RetrieveStatefulSet(ss.NameBuilder.Name(), ssNamespacedName, client)
 	if errors.IsNotFound(err) {
 		log.Info("StatefulSet: " + ssNamespacedName.Name + " not found, will create")
-		currentStatefulSet = NewStatefulSetForCR(customResource)
+		currentStatefulSet = NewStatefulSetForCR(fsm.customResource)
 		firstTime = true
 	} else {
 		log.Info("StatefulSet: " + currentStatefulSet.Name + " found")
+		//update statefulset with customer resource
+		if reconciler.ProcessAddressSettings(fsm.customResource, fsm.prevCustomResource, client) {
+			log.Info("There are new address settings change in the cr, creating a new pod template to update")
+			*fsm.prevCustomResource = *fsm.customResource
+			currentStatefulSet.Spec.Template = NewPodTemplateSpecForCR(fsm.customResource)
+		}
 	}
 
 	headlessServiceDefinition := svc.NewHeadlessServiceForCR(ssNamespacedName, serviceports.GetDefaultPorts())
@@ -225,6 +234,28 @@ func (reconciler *ActiveMQArtemisReconciler) ProcessDeploymentPlan(customResourc
 	return reconciler.statefulSetUpdates
 }
 
+func (reconciler *ActiveMQArtemisReconciler) ProcessAddressSettings(customResource *brokerv2alpha3.ActiveMQArtemis, prevCustomResource *brokerv2alpha3.ActiveMQArtemis, client client.Client) bool {
+
+	var log = logf.Log.WithName("controller_v2alpha2activemqartemis")
+	log.Info("Process addresssettings")
+
+	if len(customResource.Spec.AddressSettings.AddressSetting) == 0 {
+		return false
+	}
+
+	//we need to compare old with new and update if they are different.
+	return compareAddressSettings(&prevCustomResource.Spec.AddressSettings, &customResource.Spec.AddressSettings)
+}
+
+//returns true if currentAddressSettings need update
+func compareAddressSettings(currentAddressSettings *brokerv2alpha3.AddressSettingsType, newAddressSettings *brokerv2alpha3.AddressSettingsType) bool {
+
+	if len((*currentAddressSettings).AddressSetting) != len((*newAddressSettings).AddressSetting) || *(*currentAddressSettings).ApplyRule != *(*newAddressSettings).ApplyRule || !config.IsEqual((*currentAddressSettings).AddressSetting, (*newAddressSettings).AddressSetting) {
+		return true
+	}
+	return false
+}
+
 func (reconciler *ActiveMQArtemisReconciler) ProcessAcceptorsAndConnectors(customResource *brokerv2alpha3.ActiveMQArtemis, client client.Client, scheme *runtime.Scheme, currentStatefulSet *appsv1.StatefulSet) uint32 {
 
 	var retVal uint32 = statefulSetNotUpdated
@@ -662,7 +693,7 @@ func configureConsoleExposure(customResource *brokerv2alpha3.ActiveMQArtemis, cl
 			}
 		} else {
 			log.Info("Environment is not OpenShift, creating ingress")
-			ingressDefinition := ingresses.NewIngressForCR(namespacedName, "wconsj")
+			ingressDefinition := ingresses.NewIngressForCR(namespacedName, serviceRoutelabels, targetServiceName, targetPortName)
 			ingressNamespacedName := types.NamespacedName{
 				Name:      ingressDefinition.Name,
 				Namespace: customResource.Namespace,
@@ -1226,15 +1257,27 @@ func getDeployedResources(instance *brokerv2alpha3.ActiveMQArtemis, client clien
 	var log = logf.Log.WithName("controller_v2alpha3activemqartemis")
 
 	reader := read.New(client).WithNamespace(instance.Namespace).WithOwnerObject(instance)
-	resourceMap, err := reader.ListAll(
-		&corev1.PersistentVolumeClaimList{},
-		&corev1.ServiceList{},
-		&appsv1.StatefulSetList{},
-		&routev1.RouteList{},
-		&corev1.SecretList{},
-	)
+	var resourceMap map[reflect.Type][]resource.KubernetesResource
+	var err error
+	if isOpenshift, _ := environments.DetectOpenshift(); isOpenshift {
+		resourceMap, err = reader.ListAll(
+			&corev1.PersistentVolumeClaimList{},
+			&corev1.ServiceList{},
+			&appsv1.StatefulSetList{},
+			&routev1.RouteList{},
+			&corev1.SecretList{},
+		)
+	} else {
+		resourceMap, err = reader.ListAll(
+			&corev1.PersistentVolumeClaimList{},
+			&corev1.ServiceList{},
+			&appsv1.StatefulSetList{},
+			&extv1b1.IngressList{},
+			&corev1.SecretList{},
+		)
+	}
 	if err != nil {
-		log.Error(err, "Failed to list deployed objects. ", err)
+		log.Error(err, "Failed to list deployed objects. ")
 		return nil, err
 	}
 
@@ -1381,11 +1424,11 @@ func NewPodTemplateSpecForCR(customResource *brokerv2alpha3.ActiveMQArtemis) cor
 		}
 		reqLogger.V(1).Info("Process addresssetting", "ApplyRule", *envVarApplyRuleValue)
 
-		brokerYaml := cr2jinja2.MakeBrokerCfgOverrides(customResource, nil, nil)
+		brokerYaml := cr2jinja2v2alpha3.MakeBrokerCfgOverrides(customResource, nil, nil)
 		InitContainers := []corev1.Container{
 			{
 				Name:            "amq-broker-init",
-				Image:           "registry.redhat.io/amq7/amq-broker-init-rhel7:0.2",
+				Image:           "quay.io/artemiscloud/activemq-artemis-broker-init:0.2.1",
 				ImagePullPolicy: "Always",
 				Command:         []string{"/bin/bash"},
 				Args: []string{"-c",
